from unittest import TestCase

from mock import patch

from hestia.logging_utils import log_spec, DATETIME_REGEX


class TestLoggingUtils(TestCase):
    def test_has_timestamp(self):
        self.assertEqual(
            log_spec(log_line='foo', timestamp='2018-12-11 10:24:57 UTC'),
            '2018-12-11 10:24:57 UTC -- foo'
        )

    def test_has_no_timestamp(self):
        with patch('hestia.logging_utils.now') as now_patch:
            log_spec(log_line='foo')

        self.assertEqual(now_patch.call_count, 1)

    def test_log_line_has_datetime(self):
        self.assertEqual(
            log_spec(log_line='2018-12-11 10:24:57 UTC foo', check_timestamp=True),
            '2018-12-11 10:24:57 UTC -- foo'
        )

    def test_log_line_has_iso_datetime(self):
        self.assertEqual(
            log_spec(log_line='2018-12-11T08:49:07.163495183Z foo', check_timestamp=True),
            '2018-12-11 08:49:07 UTC -- foo'
        )

    def test_log_line_has_level(self):
        self.assertEqual(
            log_spec(log_line='2018-12-11T08:49:07.163495183Z foo',
                     check_timestamp=True,
                     log_level='INFO'),
            '2018-12-11 08:49:07 UTC INFO -- foo'
        )



from unittest import TestCase

from hestia.units import to_percentage


class ToPercentageTest(TestCase):
    """A test case for the `to_percentage`."""
    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (0, '0%'),
            (0.25, '25%'),
            (-0.25, '-25%'),
            (12, '1200%'),
            (0.123, '12.3%'),
            (0.12345, '12.35%'),
            (0.12001, '12%'),
            (0.12101, '12.1%'),
            ('0', '0%'),
            ('0.25', '25%'),
        ]
        for value, expected in test_data:
            result = to_percentage(value)
            self.assertEqual(result, expected)

    def test_raises_value_error_for_invalid_types(self):
        with self.assertRaises(ValueError):
            to_percentage('foo')



class AuthenticationError(Exception):
    pass


class AuthenticationTypes(object):
    TOKEN = 'Token'
    INTERNAL_TOKEN = 'Internaltoken'
    EPHEMERAL_TOKEN = 'EphemeralToken'

    VALUES = {TOKEN, INTERNAL_TOKEN, EPHEMERAL_TOKEN}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import datetime

from unittest import TestCase

from hestia.tz_utils import local_now
from hestia.humanize import humanize_timedelta, humanize_timesince


class HumanizeTimesinceTest(TestCase):
    """A test case for humanize timesince"""

    def test_humanize_timesince(self):
        self.assertEqual(humanize_timesince(local_now()), 'a few seconds ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(minutes=1)),
                         '1 minute ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(minutes=10)),
                         '10 minutes ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=1)),
                         '1 hour ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=10)),
                         '10 hours ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=24)),
                         '1 day ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=72)),
                         '3 days ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(hours=168)),
                         '1 week ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=1)),
                         '1 week ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=3)),
                         '3 weeks ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(weeks=53)),
                         '1 year ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(days=365)),
                         '1 year ago')
        self.assertEqual(humanize_timesince(local_now() - datetime.timedelta(days=800)),
                         '2 years ago')

    def test_humanize_times_in_the_future(self):
        self.assertEqual(humanize_timesince(local_now() + datetime.timedelta(minutes=1)),
                         'a few seconds ago')

    def test_humanize_timesince_few_seconds(self):
        self.assertEqual(u'Last update: ' + humanize_timesince(local_now()),
                         u'Last update: a few seconds ago')


class HumanizeTimeDeltaTest(TestCase):
    """A test case for the `humanize_timedelta`."""
    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (7200, '2h'),
            (36, '36s'),
            (3600, '1h'),
            (3800, '1h 3m'),
            (33000, '9h 10m'),
            (720000, '8d 8h'),
            (1000000, '11d 13h 46m'),
        ]
        for value, expected in test_data:
            result = humanize_timedelta(value)
            self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from hestia.tz_utils import local_now


def humanize_timesince(start_time):
    """Creates a string representation of time since the given `start_time`."""
    if not start_time:
        return start_time

    delta = local_now() - start_time

    # assumption: negative delta values originate from clock
    #             differences on different app server machines
    if delta.total_seconds() < 0:
        return 'a few seconds ago'

    num_years = delta.days // 365
    if num_years > 0:
        return '{} year{} ago'.format(
            *((num_years, 's') if num_years > 1 else (num_years, '')))

    num_weeks = delta.days // 7
    if num_weeks > 0:
        return '{} week{} ago'.format(
            *((num_weeks, 's') if num_weeks > 1 else (num_weeks, '')))

    num_days = delta.days
    if num_days > 0:
        return '{} day{} ago'.format(
            *((num_days, 's') if num_days > 1 else (num_days, '')))

    num_hours = delta.seconds // 3600
    if num_hours > 0:
        return '{} hour{} ago'.format(*((num_hours, 's') if num_hours > 1 else (num_hours, '')))

    num_minutes = delta.seconds // 60
    if num_minutes > 0:
        return '{} minute{} ago'.format(
            *((num_minutes, 's') if num_minutes > 1 else (num_minutes, '')))

    return 'a few seconds ago'


def humanize_timedelta(seconds):
    """Creates a string representation of timedelta."""
    hours, remainder = divmod(seconds, 3600)
    days, hours = divmod(hours, 24)
    minutes, seconds = divmod(remainder, 60)

    if days:
        result = '{}d'.format(days)
        if hours:
            result += ' {}h'.format(hours)
        if minutes:
            result += ' {}m'.format(minutes)
        return result

    if hours:
        result = '{}h'.format(hours)
        if minutes:
            result += ' {}m'.format(minutes)
        return result

    if minutes:
        result = '{}m'.format(minutes)
        if seconds:
            result += ' {}s'.format(seconds)
        return result

    return '{}s'.format(seconds)
